//
//  ViewController.swift
//  VoltageInput
//
//  Created by Phidgets on 2018-01-24.
//  Copyright © 2018 Phidgets. All rights reserved.
//

import UIKit
import Phidget22Swift

class ViewController: UIViewController {

    let ch = VoltageInput()
    var phidgetInfoBox:PhidgetInfoBoxViewController! = nil
    var pickerArray = [String]()
    
    @IBOutlet var sensorValueTitle: UILabel!
    @IBOutlet var sensorValueLabel: UILabel!
    @IBOutlet var voltageLabel: UILabel!
    @IBOutlet var dataIntervalSlider: UISlider!
    @IBOutlet var dataIntervalLabel: UILabel!
    @IBOutlet var changeTriggerSlider: UISlider!
    @IBOutlet var changeTriggerLabel: UILabel!
    @IBOutlet var voltageRangeButton: UIButton!
    @IBOutlet var sensorTypeButton: UIButton!
    @IBOutlet var stackView: UIStackView!
    
    var voltageRangeArray = [String]()
    var sensorTypeArray =  [String]()
    
    override func viewDidLoad() {
        super.viewDidLoad()
        do {
            try Net.enableServerDiscovery(serverType: .deviceRemote)
            let _ = ch.attach.addHandler(attach_handler)
            let _ = ch.detach.addHandler(detach_handler)
            let _ = ch.error.addHandler(error_handler)
            let _ = ch.voltageChange.addHandler(voltage_change)
            let _ = ch.sensorChange.addHandler(sensor_change)
            try ch.open()
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }

    //MARK: Event Handlers
    func attach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: sender)
            let deviceID = try sender.getDeviceID()
            let channelSubClass = try sender.getChannelSubclass()
            let attachedDevice = sender as! VoltageInput
            
            DispatchQueue.main.async {
                self.stackView.isHidden = false
                self.sensorTypeButton.isHidden = true
                self.sensorValueTitle.isHidden = true
                self.sensorValueLabel.isHidden = true
            }
            
            let minDataInterval = try Float(attachedDevice.getMinDataInterval())
            let maxDataInterval = try Float(attachedDevice.getMaxDataInterval())
            let dataInterval = try Float(attachedDevice.getDataInterval())
            let minVoltageChangeTrigger = try attachedDevice.getMinVoltageChangeTrigger()
            let maxVoltageChangeTrigger = try attachedDevice.getMaxVoltageChangeTrigger()
            let voltageChangeTrigger = try attachedDevice.getVoltageChangeTrigger()
            
            DispatchQueue.main.async {
                self.dataIntervalSlider.minimumValue = minDataInterval
                self.dataIntervalSlider.maximumValue = maxDataInterval
                self.dataIntervalSlider.value = dataInterval
                self.dataIntervalLabel.text = "\(dataInterval) ms"
                
                //if min/max change trigger is equal, change trigger is not configurable
                if minVoltageChangeTrigger == maxVoltageChangeTrigger{
                    self.changeTriggerSlider.isEnabled = false
                }
                else{
                    self.changeTriggerSlider.minimumValue = Float(minVoltageChangeTrigger)
                    self.changeTriggerSlider.maximumValue = Float(maxVoltageChangeTrigger)
                    self.changeTriggerSlider.value = Float(voltageChangeTrigger)
                    self.changeTriggerLabel.text = "\(voltageChangeTrigger) V"
                }
                
                if(deviceID == DeviceID.PN_ADP1000){
                    self.voltageRangeArray.removeAll()
                    self.voltageRangeArray = ["400 mV", "2 V"]
                }
                else if(deviceID == DeviceID.PN_VCP1000){
                    self.voltageRangeArray.removeAll()
                    self.voltageRangeArray = ["312.5 mV","40 V"]
                }
                else if(deviceID == DeviceID.PN_VCP1001){
                    self.voltageRangeArray.removeAll()
                    self.voltageRangeArray = ["5 V","15 V","40 V","Automatic"]
                }
                else if(deviceID == DeviceID.PN_VCP1002){
                    self.voltageRangeArray.removeAll()
                    self.voltageRangeArray = ["10 mV","40 mV","200 mV","1000 mV","Automatic"]
                }
                if(channelSubClass == ChannelSubclass.voltageInputSensorPort){
                    self.sensorTypeArray.removeAll()
                    self.sensorTypeArray = ["Generic Voltage Sensor","1114 - Temperature Sensor","1117 - Voltage Sensor","1123 - Precision Voltage Sensor","1127 - Precision Light Sensor","1130 - pH Adapter","1130 - ORP Adapter","1132 - 4-20mA Adapter","1133 - Sound Sensor","1135 - Precision Voltage Sensor","1142 - Light Sensor 1000 lux","1143 - Light Sensor 70000 lux","3500 - AC Current Sensor 10Amp lux","3501 - AC Current Sensor 25Amp","3502 - AC Current Sensor 50Amp","3503 - AC Current Sensor 100Amp","3507 - AC Voltage Sensor 0-250V (50 Hz)","3508 - AC Voltage Sensor 0-250V (60 Hz)","3509 - DC Voltage Sensor 0-200V","3510 - DC Voltage Sensor 0-75V","3511 - DC Current Sensor 0-10mA","3512 - DC Current Sensor 0-100mA","3513 - DC Current Sensor 0-1A","3514 - AC Active Power Sensor 0-250V*0-30A (50Hz)","3515 - AC Active Power Sensor 0-250V*0-30A (60Hz)","3516 - AC Active Power Sensor 0-250V*0-5A (50Hz)","3517 - AC Active Power Sensor 0-250V*0-5A (60Hz)","3518 - AC Active Power Sensor 0-110V*0-5A (60Hz)","3519 - AC Active Power Sensor 0-110V*0-15A (60Hz)","3584 - 0-50A DC Current Transducer","3585 - 0-100A DC Current Transducer","3586 - 0-250A DC Current Transducer","3587 - +-50A DC Current Transducer","3588 - +-100A DC Current Transducer","3589 - +-250A DC Current Transducer"]
                    self.sensorTypeButton.isHidden = false
                    self.sensorValueTitle.isHidden = false
                    self.sensorValueLabel.isHidden = false
                }
                
                self.voltageRangeButton.isHidden = (self.voltageRangeArray.count == 0) ?  true :  false
            }
            
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    func detach_handler(sender: Phidget){
        do{
            try phidgetInfoBox.fillPhidgetInfo(phid: nil)
            DispatchQueue.main.async {
                self.stackView.isHidden = true
            }
            
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    
    func error_handler(sender: Phidget, data: (code: ErrorEventCode, description: String)){
        self.outputErrorMessage(errorTitle: "Error Code: \(data.code)", errorMessage: data.description)
    }
    var timeStamp:TimeInterval = 0
    var lastTimeStamp:TimeInterval = 0
    func outputErrorMessage(errorTitle:String, errorMessage:String){
        timeStamp = NSDate.timeIntervalSinceReferenceDate
        //Add a delay for error events so the user does not get bombarded by error events
        if(timeStamp - lastTimeStamp > 5){
            let alert = UIAlertController(title: errorTitle, message: errorMessage, preferredStyle: .alert)
            alert.addAction(UIAlertAction(title: "OK", style: .default, handler: {action -> Void in
            }))
            self.present(alert, animated: true, completion: nil)
            lastTimeStamp = timeStamp
        }
    }
    
    func sensor_change(sender: VoltageInput, data: (sensorValue: Double, sensorUnit: UnitInfo)){
        DispatchQueue.main.async {
            self.sensorValueLabel.text = String(format:"%.3f%@", data.sensorValue, data.sensorUnit.symbol)
        }
    }
    
    func voltage_change(sender: VoltageInput, voltage: Double){
        DispatchQueue.main.async {
            self.voltageLabel.text = String(format:"%.4fV",voltage)
        }
    }

    
    //MARK: navigation
    @IBAction func pickerExit(_ segue: UIStoryboardSegue){
        dimBackground()
    }
    
    override func prepare(for segue: UIStoryboardSegue, sender: Any?) {
        if segue.identifier == "infoBoxSegue" {
            phidgetInfoBox = segue.destination as! PhidgetInfoBoxViewController
        }
        else if segue.identifier == "voltageRangeSegue"{
            let vc:PickerViewController = segue.destination as! PickerViewController
            dimBackground()
            vc.titleName = "Voltage Range"
            vc.pickerArray = voltageRangeArray
            vc.ch = ch
        }
        else if segue.identifier == "sensorTypeSegue"{
            let vc:PickerViewController = segue.destination as! PickerViewController
            dimBackground()
            vc.titleName = "Sensor Type"
            vc.pickerArray = sensorTypeArray
            vc.ch = ch
        }
    }
    
    //MARK: GUI controls
    @IBAction func setDataInterval(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setDataInterval(UInt32(slider.value))
            dataIntervalLabel.text = "\(UInt32(slider.value)) ms"
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
    @IBAction func setChangeTrigger(_ sender: Any) {
        let slider = sender as! UISlider
        do{
            try ch.setVoltageChangeTrigger(Double(slider.value))
            changeTriggerLabel.text = String(format: "%0.4fV", slider.value)
        } catch let err as PhidgetError {
            outputErrorMessage(errorTitle: "Error Code: \(err.errorCode)", errorMessage: err.description)
        } catch {
            print (error)
        }
    }
 
    //MARK: Misc
    override func didReceiveMemoryWarning() {
        super.didReceiveMemoryWarning()
    }
    
    func dimBackground(){
        self.view.alpha = (self.view.alpha == 1.0) ? 0.5 : 1.0
    }
}

